//
//   TUMME 3.1  is licensed under the Apache License, Version 2.0 (the "License");
//   you may not use TUMME – version 3.1 except in compliance with the License.
//   You may obtain a copy of the License at
//       http://www.apache.org/licenses/LICENSE-2.0
//   The license is also given in the LICENSE file.
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and limitations under the License.
//
// -------------------------------------------------------------------------------------------
// 
//   TUMME 3.1
//           
//   Rui Ming Zhang
//     
//   Center for Combustion Energy, Department of Energy and Power
//   Engineering, and Key Laboratory for Thermal Science and Power 
//   Engineering of Ministry of Education, Tsinghua University, Beijing
//   
//   Department of Chemistry, Chemical Theory Center, and Minnesota 
//   Supercomputing Institute, University of Minnesota, Minneapolis, 
//   Minnesota
//                  
//   Xuefei Xu
//
//   Center for Combustion Energy, Department of Energy and Power
//   Engineering, and Key Laboratory for Thermal Science and Power 
//   Engineering of Ministry of Education, Tsinghua University, Beijing
//
//   Donald G. Truhlar
// 
//   Department of Chemistry, Chemical Theory Center, and Minnesota 
//   Supercomputing Institute, University of Minnesota, Minneapolis, 
//   Minnesota
//  
//   The master equation code includes multi-structural variational 
//   transition state theory with small-curvature tunneling (MS-VTST/SCT)
//   for calculating the microcanonical flux coefficients needed as input
//   to the master equation. The TUMME program can do double, quadruple, 
//   and octuple precision, it can execute with MP or MPI parallel schemes, 
//   and it can interface to Gaussian, Polyrate, Gaussrate, and/or MSTor 
//   output files. Report bugs to: zrm17@tsinghua.org.cn
// 
//   Copyright 2020~2023
//   Donald G. Truhalar and Regents of the University of Minnesota
//   Xuefei Xu and the Tsinghua University 
//

#ifndef __INTERFACE_DD_H
#define __INTERFACE_DD_H

#include <mblas_dd.h>
#include <mlapack_dd.h>
#include <stdint.h>
#include <omp.h>
#include <qd/dd_real.h>
#include <math.h>
#include <iostream>
#include <string.h>

typedef struct _well    * WellStack; 
typedef struct _kE      * kEStack;
typedef struct _dk      * dkStack;
typedef struct _well    * Well;
typedef struct _param   * Param;
typedef struct _pseubim * Pseubim;

struct _well
{
    double* dos;
    int32_t dos_sz;
    double* dos_t;
    double  E0;
    double  E0_t;
    int32_t size;
    double  Z;
};

struct _kE
{
    double* kE;
    int32_t kE_sz;
    int32_t ph1_index;
    int32_t ph2_index;
};

struct _dk
{
    double* dk;
    int32_t dk_sz;
    int32_t ph1_index;
    int32_t ph2_index;
};

struct _param
{
    double slope;
    double EdConst;
    double Tc;
    double exponent;
    double dE;
    double ener_step;
    double ener_ref;
    double beta;
};

struct _pseubim
{
    int32_t index;
    double  excessconc;
};


extern "C"
{
void ME_dd_Main( 
    int32_t ompnumthread, 
    WellStack well_stack,   int32_t Nwell, 
    kEStack kE_stack,       int32_t kE_stack_sz,
    Param param,            int32_t Nph,  
    double *evals,          double *evecs );

void pseuME_dd_Main( 
    int32_t ompnumthread, 
    WellStack well_stack,   int32_t Nwell, 
    kEStack kE_stack,       int32_t kE_stack_sz,
    dkStack dk_stack,       int32_t dk_stack_sz,
    Param param,            int32_t Nph,  
    Pseubim pseubim_stack,  int32_t Npseubim,
    double *evals,          double *evecs );

}

#endif
