//
//   TUMME 3.1  is licensed under the Apache License, Version 2.0 (the "License");
//   you may not use TUMME – version 3.1 except in compliance with the License.
//   You may obtain a copy of the License at
//       http://www.apache.org/licenses/LICENSE-2.0
//   The license is also given in the LICENSE file.
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and limitations under the License.
//
// -------------------------------------------------------------------------------------------
// 
//   TUMME 3.1
//           
//   Rui Ming Zhang
//     
//   Center for Combustion Energy, Department of Energy and Power
//   Engineering, and Key Laboratory for Thermal Science and Power 
//   Engineering of Ministry of Education, Tsinghua University, Beijing
//   
//   Department of Chemistry, Chemical Theory Center, and Minnesota 
//   Supercomputing Institute, University of Minnesota, Minneapolis, 
//   Minnesota
//                  
//   Xuefei Xu
//
//   Center for Combustion Energy, Department of Energy and Power
//   Engineering, and Key Laboratory for Thermal Science and Power 
//   Engineering of Ministry of Education, Tsinghua University, Beijing
//
//   Donald G. Truhlar
// 
//   Department of Chemistry, Chemical Theory Center, and Minnesota 
//   Supercomputing Institute, University of Minnesota, Minneapolis, 
//   Minnesota
//  
//   The master equation code includes multi-structural variational 
//   transition state theory with small-curvature tunneling (MS-VTST/SCT)
//   for calculating the microcanonical flux coefficients needed as input
//   to the master equation. The TUMME program can do double, quadruple, 
//   and octuple precision, it can execute with MP or MPI parallel schemes, 
//   and it can interface to Gaussian, Polyrate, Gaussrate, and/or MSTor 
//   output files. Report bugs to: zrm17@tsinghua.org.cn
// 
//   Copyright 2020~2023
//   Donald G. Truhalar and Regents of the University of Minnesota
//   Xuefei Xu and the Tsinghua University 
//

#ifndef __MEDD_H
#define __MEDD_H

#include "Interface_dd.h"

#define kin_mat_dd(i,j) kin_mat_dd[ (j) + (i)*lda ]
#define kernel_dd(i,j) kernel_dd[ (j) + (i)*lda ]
#define kernel_T_dd(i,j) kernel_T_dd[(j) + (i)*lda]
#define kernel_dd_rev(i,j) kernel_dd_rev[(j) + (i)*lda]


void Sym_Eig(dd_real* &A, dd_real* &w, const int32_t  &ndim);
void Trunc_Transpose(dd_real* &kernel_T_dd, const int32_t &index_trunc, const int32_t &mat_size,dd_real* &kernel_dd);
void Creat_Kernel(Well &well,Param &param,dd_real* &kernel_dd);
void Trunc_W_DoS(Well &well,Param &param);
void array_add(double* &cum,double* &a,int32_t &size);
void ME_dd_Solver(
                    WellStack &well_stack,      Param &param, 
                    int32_t* &well_offset,      const int32_t &well_sz,
                    const int32_t &Nph,         const int32_t &mat_size,
                    dd_real** &kernel_stack_dd, double** &kE_map,
                    double* &evals,             double* &evecs);
void  pseuME_dd_Solver(
                    WellStack &well_stack,      Param &param, 
                    Pseubim &Pseubim_stack,     int32_t* &well_offset,     
                    const int32_t &Nwell,       const int32_t &Nph,
                    const int32_t &Npseubim,    const int32_t &mat_size,  
                    dd_real** &kernel_stack_dd,
                    double** &kE_map,           double** &dk_map,           
                    double* &evals,             double* &evecs);

#endif